@@ -83,7 +83,21 @@ class Handler(ABC):
         format string contains a '%' which indicates where the placeholder goes.
         """
 
+    @property
+    def generic_url(self) -> typing.Optional[str]:
+        """ A generic URL that can be used for login irrespective of identity """
+        return None
+
     @property
     @abstractmethod
     def description(self) -> str:
         """ Returns a description of the service, in HTML format. """
+
+    @property
+    def logo_html(self) -> typing.Optional[str]:
+        """ A list of tuples of (html,label) for the login buttons.
+
+        The HTML should be an isolated <svg> element, or an <img src> pointing to a publicly-usable
+        https: URL.
+        """
+        return None
