@@ -7,7 +7,7 @@ import urllib.parse
 import requests
 from requests_oauthlib import OAuth1, OAuth1Session
 
-from .. import disposition
+from .. import disposition, utils
 from . import Handler
 
 
@@ -29,6 +29,10 @@ class Twitter(Handler):
     def url_schemes(self):
         return [('https://twitter.com/%', 'username')]
 
+    @property
+    def logo_html(self):
+        return [(utils.read_icon("twitter.svg"), 'Twitter')]
+
     def __init__(self, client_key: str,
                  client_secret: str,
                  timeout: int = None,
@@ -127,6 +131,10 @@ class Twitter(Handler):
             redir,
             user_info)
 
+    @property
+    def generic_url(self):
+        return 'https://twitter.com/'
+
 
 def from_config(config, storage):
     """ Generate a Twitter handler from the given config dictionary.
